;KEYBUK.ASM by Jonathan McDowell.
;Copyright 1995 Jonathan McDowell for Project Purple;
;Replacement for KEYB UK. Uses 256 bytes of memory.
;22/01/95 - Converted from UKKEYB by Julian V. Moss
;09/03/95 - Changed to relocate at 60h, over PSP.
;18/03/95 - Changed to use tables to check scan codes.
;10/04/99 - Modified to compile with NASM (Previously A86)
;           for release to the outside world. v1.0.0

                org 100h

Main:           jmp Install             ;Install as a TSR

TSRSt:          sti
                pushf
                push ax
                push ds
                push bx

                mov bx,ShiftedScanCodes-0A3h
                mov ax, 0040h
                mov ds, ax              ;Set DS = 0040H
                mov al, [0017h]         ;Read shift state
                and al, 3h
                jnz ReadKey
                mov bx,NormalScanCodes-0A3h

ReadKey:        in al, 60h              ;Read key
CheckScanCode:  cmp al,[CS:bx]
                jz SHORT GotIt
                inc bx
                inc bx
                cmp BYTE [CS:bx],0
                jz SHORT OldISR
                jmp SHORT CheckScanCode
GotIt:          push dx
                mov dl,[CS:bx+1]
                mov dh,al
                jmp SHORT PutBuf

ShiftedScanCodes:
                db 03h , 022h           ;Quote
                db 04h , 09Ch           ;Pound
                db 28h , 040h           ;At
                db 29h , 0AAh           ;Not
                db 2Bh , 07Eh           ;Tilde
                db 56h , 07Ch           ;Bar
                db 0

NormalScanCodes:
                db 2Bh , 023h           ;Hash
                db 56h , 05Ch           ;Backslash
                db 0

PutBuf:         in al, 61h              ;Reset keyboard
                mov ah, al
                or al, 80h
                out 61h, al
                mov al, ah
                out 61h, al
                push di
                mov bx, [001ch]         ;Get buffer tail
                mov di, bx
                inc bx
                inc bx
                cmp bx, [0082h]         ;Check for wrap-round
                jnz PutKey
                mov bx, [0080h]         ;If so, set to start
                cmp bx, [001Ah]         ;Check if buffer full
                jz BuFull
PutKey:         mov [di],dx             ;Put key value in buffer
                mov [01Ch],bx           ;Update pointer
BuFull:         pop di
                cli
                mov al, 20h             ;Re-enable interrupts
                out 20h, al

                pop dx
                pop bx
                pop ds
                pop ax
                popf
                iret

OldISR:         pop bx
                pop ds
                pop ax
                popf
OldInt9:        db 0EAh                 ;Jump to old ISR
I9Ofs           dw 0                    ;Address of old interrupt 9
I9Seg           dw 0                    ; service routine

;------ install program as interrupt service routine ------

Install:        mov ah,9
                mov dx,InstallMsg
                int 21h
                mov ax,03509h
                int 21h                 ;Get old interrupt vector 9
                mov [I9Ofs],bx
                mov [I9Seg],es
                push cs
                push cs
                pop es
                pop ds
                mov si,TSRSt
                mov di,60h
                mov cx,Install-100h
                rep movsb
                mov dx,60h              ;Set new interrupt vector
                mov ax,02509h
                int 21h
                mov es,[CS:2Ch]
                mov ah,49h
                int 21h                 ;Free enviroment block
                mov ax,03100h
                mov dx,(Install-0A3h+15)    ;Number of paragraphs to reserve
                shr dx,4                ;Shouldn't need this, but NASM complains about >>4 on above line.
                int 21h                 ;Terminate and stay resident

InstallMsg:     db 13,10,'Installing KEYBUK v1.0.0 by Jonathan McDowell.'
                db 13,10,'Copyright 1995 Project Purple.',13,10,'$'
